changeset 3232:12f41a1f8afc libavformat

Tighten up the Westwood AUD detection. Probability of random detections used to be on the order of 2^8. It is now on the order of 2^45.
author melanson
date Fri, 18 Apr 2008 17:29:58 +0000
parents 956fc24819df
children e2bdb989f7da
files westwood.c
diffstat 1 files changed, 14 insertions(+), 5 deletions(-) [+]
line wrap: on
line diff
--- a/westwood.c	Wed Apr 16 16:52:35 2008 +0000
+++ b/westwood.c	Fri Apr 18 17:29:58 2008 +0000
@@ -90,14 +90,14 @@
     /* Probabilistic content detection strategy: There is no file signature
      * so perform sanity checks on various header parameters:
      *   8000 <= sample rate (16 bits) <= 48000  ==> 40001 acceptable numbers
+     *   flags <= 0x03 (2 LSBs are used)         ==> 4 acceptable numbers
      *   compression type (8 bits) = 1 or 99     ==> 2 acceptable numbers
      * There is a total of 24 bits. The number space contains 2^24 =
      * 16777216 numbers. There are 40001 * 2 = 80002 acceptable combinations
      * of numbers. There is a 80002/16777216 = 0.48% chance of a false
      * positive.
      */
 
-    if (p->buf_size < AUD_HEADER_SIZE)
+    if (p->buf_size < AUD_HEADER_SIZE + AUD_CHUNK_PREAMBLE_SIZE)
         return 0;
 
     /* check sample rate */
@@ -105,11 +105,20 @@
     if ((field < 8000) || (field > 48000))
         return 0;
 
+    /* enforce the rule that the top 6 bits of this flags field are reserved (0);
+     * this might not be true, but enforce it until deemed unnecessary */
+    if (p->buf[10] & 0xFC)
+        return 0;
+
     /* note: only check for WS IMA (type 99) right now since there is no
      * support for type 1 */
     if (p->buf[11] != 99)
         return 0;
 
+    /* read ahead to the first audio chunk and validate the first header signature */
+    if (AV_RL32(&p->buf[16]) != AUD_CHUNK_SIGNATURE)
+        return 0;
+
     /* return 1/2 certainty since this file check is a little sketchy */
     return AVPROBE_SCORE_MAX / 2;
 }
@@ -90,14 +90,14 @@
      *   compression type (8 bits) = 1 or 99     ==> 2 acceptable numbers
-     * There is a total of 24 bits. The number space contains 2^24 =
-     * 16777216 numbers. There are 40001 * 2 = 80002 acceptable combinations
-     * of numbers. There is a 80002/16777216 = 0.48% chance of a false
-     * positive.
+     *   first audio chunk signature (32 bits)   ==> 1 acceptable number
+     * The number space contains 2^64 numbers. There are 40001 * 4 * 2 * 1 =
+     * 320008 acceptable number combinations.
      */
 